/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcsftp;

import java.io.IOException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.bc.sftp.authentication.SshAuthentication;
import org.ow2.petals.bc.sftp.authentication.SshPasswordAuthentication;
import org.ow2.petals.bc.sftp.authentication.exception.AuthenticationException;
import org.ow2.petals.bc.sftp.connection.BasicSftpClient;
import org.ow2.petals.bc.sftp.connection.SFTPConnectionInfo;
import org.ow2.petals.bc.sftp.connection.exception.ConnectionException;
import org.ow2.petals.cli.api.command.AbstractCommand;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

/**
 * The command 'bc-sftp.check-connect'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class BcSftpCheckConnect extends AbstractCommand {

    /**
     * The command name
     */
    public static final String NAME = "bc-sftp.check-connect";

    /**
     * The command description
     */
    public static final String DESCRIPTION = "Check connection establishment of the BC SFTP";

    /**
     * The option associated to the SSH server
     */
    public static final String SERVER_SHORT_OPTION = "s";

    private static final String SERVER_LONG_OPTION = "server";

    private static final String SERVER_ARG_NAME = "server";

    private static final Option SERVER_OPTION = Option.builder(SERVER_SHORT_OPTION).longOpt(SERVER_LONG_OPTION)
            .required(true).numberOfArgs(1).argName(SERVER_ARG_NAME)
            .desc("The SSH server on which the connection will be established.").build();

    /**
     * The option associated to the port of the SSH server.
     */
    public static final String PORT_SHORT_OPTION = "p";

    private static final String PORT_LONG_OPTION = "port";

    private static final String PORT_ARG_NAME = "port";

    private static final Option PORT_OPTION = Option.builder(PORT_SHORT_OPTION).numberOfArgs(1).argName(PORT_ARG_NAME)
            .required(false).longOpt(PORT_LONG_OPTION).desc("The port of the SSH server (Default value: 22)").build();

    private static final int DEFAULT_PORT = 22;

    /**
     * The option associated to the username
     */
    public static final String USER_SHORT_OPTION = "u";

    private static final String USER_LONG_OPTION = "username";

    private static final String USER_ARG_NAME = "username";

    private static final Option USER_OPTION = Option.builder(USER_SHORT_OPTION).numberOfArgs(1).argName(USER_ARG_NAME)
            .required(true).longOpt(USER_LONG_OPTION).desc("The username.").build();

    /**
     * The option associated to the password
     */
    public static final String PWD_SHORT_OPTION = "w";

    private static final String PWD_LONG_OPTION = "password";

    private static final String PWD_ARG_NAME = "password";

    private static final Option PWD_OPTION = Option.builder(PWD_SHORT_OPTION).numberOfArgs(1).argName(PWD_ARG_NAME)
            .required(true).longOpt(PWD_LONG_OPTION).desc("The password.").build();

    /**
     * The option associated to the directory
     */
    public static final String DIR_SHORT_OPTION = "d";

    private static final String DIR_LONG_OPTION = "directory";

    private static final String DIR_ARG_NAME = "directory";

    private static final String DEFAULT_DIR = "/";

    private static final Option DIR_OPTION = Option.builder(DIR_SHORT_OPTION).numberOfArgs(1).argName(DIR_ARG_NAME)
            .required(false).longOpt(DIR_LONG_OPTION).desc("The home directory (Default value: \"/\").").build();

    public BcSftpCheckConnect() {
        super(NAME);
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription(DESCRIPTION);
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }


    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(SERVER_OPTION);
        options.addOption(PORT_OPTION);
        options.addOption(USER_OPTION);
        options.addOption(PWD_OPTION);
        options.addOption(DIR_OPTION);

        return options;
    }

    @Override
    protected void doExecute(String[] args) throws CommandException {
        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            int port = DEFAULT_PORT;
            if (cli.hasOption(PORT_SHORT_OPTION)) {
                try {
                    port = Integer.parseInt(cli.getOptionValue(PORT_SHORT_OPTION));
                } catch (final NumberFormatException e) {
                    throw new CommandException(this, "Invalid port value: " + e.getMessage());
                }
            }

            final String directory;
            if (cli.hasOption(DIR_SHORT_OPTION)) {
                directory = cli.getOptionValue(DIR_SHORT_OPTION);
            } else {
                directory = DEFAULT_DIR;
            }

            final String server = cli.getOptionValue(SERVER_SHORT_OPTION);
            final String user = cli.getOptionValue(USER_SHORT_OPTION);
            final String password = cli.getOptionValue(PWD_SHORT_OPTION);

            final SshAuthentication sshAuthentication = new SshPasswordAuthentication(user,
                    password);

            final BasicSftpClient client = new BasicSftpClient(new SFTPConnectionInfo(server,
                    String.valueOf(port), sshAuthentication, directory, false));
            client.connectAndLog();
            client.configure();
            client.getConnectionState();
            client.disconnect();

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final IOException e) {
            throw new CommandException(this, e);
        } catch (final AuthenticationException e) {
            throw new CommandException(this, e);
        } catch (final ConnectionException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public boolean isConnectionRequired() {
        return false;
    }
}
