/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcsftp;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;

import org.junit.Test;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.base.junit.shell.DummyShell;

/**
 * Unit tests of the command 'bc-sftp.check-connect'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class BcSftpCheckConnectTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] {});
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, BcSftpCheckConnect.SERVER_SHORT_OPTION,
                    BcSftpCheckConnect.USER_SHORT_OPTION, BcSftpCheckConnect.PWD_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>the option associated to the server is set but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] { "-" + BcSftpCheckConnect.SERVER_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected missing argument", BcSftpCheckConnect.SERVER_SHORT_OPTION, e
                    .getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>the option associated to the port is set but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] { "-" + BcSftpCheckConnect.PORT_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected missing argument", BcSftpCheckConnect.PORT_SHORT_OPTION, e
                    .getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>the option associated to the username is set but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_4() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] { "-" + BcSftpCheckConnect.USER_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected missing argument", BcSftpCheckConnect.USER_SHORT_OPTION, e
                    .getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>the option associated to the password is set but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_5() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] { "-" + BcSftpCheckConnect.PWD_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected missing argument", BcSftpCheckConnect.PWD_SHORT_OPTION, e
                    .getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>the option associated to the directory is set but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_6() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] { "-" + BcSftpCheckConnect.DIR_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected missing argument", BcSftpCheckConnect.DIR_SHORT_OPTION, e
                    .getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test of the command "bc-sftp.check-connect" against argument management,
     * where:
     * <ul>
     * <li>the option associated to the server is set to a valid value,</li>
     * <li><b>the option associated to the port is set with an invalid value</b>,</li>
     * <li>the option associated to the username is set to a valid value,</li>
     * <li>the option associated to the password is set to a valid value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testArgumentsError_7() throws CommandException {
        final BcSftpCheckConnect command = new BcSftpCheckConnect();
        command.setShell(new DummyShell());
        try {
            command.execute(new String[] {
                    "-" + BcSftpCheckConnect.SERVER_SHORT_OPTION, "localhost",
                    "-" + BcSftpCheckConnect.PORT_SHORT_OPTION, "invalid-value",
                    "-" + BcSftpCheckConnect.USER_SHORT_OPTION, "username",
                    "-" + BcSftpCheckConnect.PWD_SHORT_OPTION, "password"});
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandException e) {
            assertTrue("Error label is missing.", e.getMessage().contains("Invalid port value"));
        }
    }

}
